from nltk.corpus import words


modern_technical_terminology = [
    "blog",
    "blogger",
    "website",
    "webpage",
    "homepage",
    "inbox",
    "inboxes",
    "hashtag",
    "email",
    "outsource",
    "timeline",
    "airfare",
    "paycheck",
    "whiteboard",
    "podcast",
    "livestream",
    "smartphone",
    "upload",
    "download",
    "listserv",
    "screensaver",
    "telecommute",
    "multitask",
    "elife",
    "infographic",
    "uppercase",
    "lowercase",
    "trademark",
    "playlist",
    "emoji",
    "earbud",
    "audiobook",
    "skillset",

    "ipad",
    "iphone",
    "google",
    "ebay",
    "itunes",
    "instagram",
    "facebook",
    "twitter",
    "istock",
    "workweek"
]
modern_social_terminology = [
    "who'd",
    "mom",
    "dad",
    "timeout",
    "bandana",
    "transgender",
    "cisgender",
    "heteronormative",
    "transvaginal",
    "descrimination",
    "anxious",
    "handwash",
    "restroom",
    "sanitizer",
    "misappropriate",
    "dystopian",
    "scrunchy",
    "rehab",
    "veggie",
    "prep",
    "indoctrinate"
    "attune",
    "upcycling",
    "sedate",
    "badass",
    "pushback",
    "inundate",
    "overshare",
    "cilantro",
    "reframing",
    "midcentury",
    "flatline",
    "weeknight",
    "prenup",
    "multigenerational",
    "microfiber",
    "agent",
    "caregiver",
    "pushup",
    "moisturizer",
    "bedsheet",
    "spritz",
    "photojournalist",
    "allergy",
    "granola",
    "hotcakes",
    "breadsticks",
    "counterintuitive",
    "meth",
    "foodie",
    "takeout",
    "kombucha",
    "skydive",
    "goop",
    "groundbreaking",
    "spandex",
    "cardio",
    "popsicle",
    "walkup",
    "townhouse",
    "condo",
    "zit",
    "eyeroll",
    "pilates",
    "ultrasound",
    "cesarean",
    "sweatpants",
    "paralegal",
    "gatekeep",
    "meetup",
    "waitstaff",
    "clutter",
    "zipper",
    "deadbolt",
    "hoagie",
    "surefooted",
    "racecar",
    "smurf",
    "humongous",
    "scattershot",
    "debuting",
    "vegan",
    "yearslong"
]

adopted_words = [
    "reiki",
    'mâché',
    'guac',
    'queso',
    'nacho',
    'quesadilla',
    'mitzvah',
    'naïve',
    'fiancée'
    'à',
    'latte',
    'blasé',
    'purée',
    'tchotchke',
    'décor',
    'bébé',
    'cliché',
    'doppelgänger',
    'kitschy',
    'risqué',
    'schlep',
    'pâtés',
    'piñata',
    'lasagna',
    'quinceañera',

]

dictionary = set(words.words() + modern_technical_terminology + modern_social_terminology + adopted_words)
